@@ -1,8 +1,11 @@
 import re
 import os
 import os.path
+import shutil
+import uuid
 import tempfile
 import subprocess
+from django.db import transaction
 from edge.models import Fragment, Genome
 from edge.blast import BLAST_DB, default_genome_db_name
 from edge.blast import Blast_Accession
@@ -29,6 +32,17 @@ def fragment_fasta_fn(fragment):
     )
 
 
+def does_blast_db_have_all_fragments(fragments, dbname):
+    f = open(dbname + ".nsd")
+    lines = f.readlines()
+    f.close()
+    print("verifying: expecting %s fragments, got %s lines in nsd" % (len(fragments), len(lines)))
+    return len(fragments) * 2 == len(lines)
+
+
+# needs to be in atomic block, so we can build fragment location indices, if
+# needed
+@transaction.atomic()
 def build_fragment_fasta(fragment, refresh=False):
     fn = fragment_fasta_fn(fragment)
     make_required_dirs(fn)
@@ -42,25 +56,25 @@ def build_fragment_fasta(fragment, refresh=False):
         sequence = re.sub(r"[^agctnAGCTN]", "n", sequence)
         if fragment.circular is True:
             sequence = sequence + sequence
-        f = open(fn, "w")
-        f.write(
-            ">gnl|edge|%s %s\n%s\n"
-            % (Blast_Accession.make(fragment), fragment.name, sequence)
-        )
-        f.close()
+
+        # writing first to a temp file, rename to an expected file location,
+        # prevents possible race condition of accessing/writing to the same
+        # file (which may or may not be a problem on a NAS, i don't know).
+        with tempfile.NamedTemporaryFile(mode="w", delete=False) as tmpf:
+            tmpf.write(
+                ">gnl|edge|%s %s\n%s\n"
+                % (Blast_Accession.make(fragment), fragment.name, sequence)
+            )
+        # i think this will write atomically to dest, and copies over different fs
+        shutil.move(tmpf.name, fn)
+
     return fn
 
 
-def build_db(fragments, dbname, refresh=True):
+def build_db(fragments, dbname, refresh=True, attempt=0):
     if len(fragments) == 0:
         return None
 
-    if refresh is False and (
-        os.path.isfile(dbname + ".nal") or os.path.isfile(dbname + ".nsq")
-    ):
-        print("already built %s" % dbname)
-        return dbname
-
     fns = []
     for fragment in fragments:
         fn = build_fragment_fasta(fragment, refresh)
@@ -74,6 +88,11 @@ def build_db(fragments, dbname, refresh=True):
                 for line in inf:
                     f.write(line)
 
+    # the following prevents concurrent blastdb builds corrupting each other
+    orig_dbname = dbname
+    unique_suffix = str(uuid.uuid4())
+    dbname = "%s_%s" % (dbname, unique_suffix)
+
     print("building blast db %s" % dbname)
     make_required_dirs(dbname)
     cmd = "%s/makeblastdb -in %s -out %s " % (settings.NCBI_BIN_DIR, fafile, dbname)
@@ -84,14 +103,24 @@ def build_db(fragments, dbname, refresh=True):
         print(r)
 
     os.unlink(fafile)
+
+    if not does_blast_db_have_all_fragments(fragments, dbname) and attempt < 5:
+        print("does not have all fragments, retry (attempts: %s)" % (attempt + 1,))
+        return build_db(fragments, orig_dbname, refresh=refresh, attempt=(attempt + 1))
+
     return dbname
 
 
 def build_genome_db(genome, refresh=False):
-    fragments = list(genome.fragments.all())
-    dbname = build_db(fragments, default_genome_db_name(genome), refresh=refresh)
-    genome.blastdb = dbname
-    genome.save()
+    if genome.blastdb is None or refresh:
+        fragments = list(genome.fragments.all())
+        dbname = build_db(fragments, default_genome_db_name(genome), refresh=refresh)
+        genome.blastdb = dbname
+        genome.save()
+        return dbname
+    else:
+        print("already built genome blast db for %s" % genome.id)
+        return genome.blastdb
 
 
 def check_and_build_genome_db(genome, refresh=False):
