@@ -421,6 +421,7 @@ class GenomeRecombinationTest(TestCase):
         g = self.build_genome(False, template)
         self.assertEquals(Operation.objects.count(), 0)
         c = recombine(g, cassette, arm_len)
+
         self.assertEquals(Operation.objects.count(), 1)
         self.assertEquals(c.operation_set.all()[0].type, Operation.RECOMBINATION[0])
         self.assertEquals(
@@ -428,6 +429,26 @@ class GenomeRecombinationTest(TestCase):
             json.dumps(dict(cassette=cassette, homology_arm_length=arm_len)),
         )
 
+    def test_inserting_new_sequence_without_removal_of_old_sequence(self):
+        upstream = "gagattgtccgcgtttt"
+        front_bs = "catagcgcacaggacgcggag"
+        back_bs = "taatgaccccgaagcagg"
+        downstream = "gttaaggcgcgaacat"
+        replaced = "a" * 100
+
+        template = "".join([upstream, front_bs, back_bs, downstream])
+        cassette = "".join([front_bs, replaced, back_bs])
+
+        arm_len = min(len(front_bs), len(back_bs))
+        g = self.build_genome(False, template)
+        c = recombine(g, cassette, arm_len)
+
+        self.assertNotEqual(g.id, c.id)
+        self.assertEquals(
+            c.fragments.all()[0].indexed_fragment().sequence,
+            "".join([upstream, cassette, downstream]),
+        )
+
     def test_annotates_cassette(self):
         upstream = "gagattgtccgcgtttt"
         front_bs = "catagcgcacaggacgcggag"
@@ -448,10 +469,10 @@ class GenomeRecombinationTest(TestCase):
 
         a = c.fragments.all()[0].indexed_fragment().annotations()
         self.assertEquals(len(a), 1)
-        self.assertEquals(a[0].base_first, len(upstream) + 1)
-        self.assertEquals(a[0].base_last, len(upstream + cassette))
+        self.assertEquals(a[0].base_first, len(upstream) + len(front_bs) + 1)
+        self.assertEquals(a[0].base_last, len(upstream + front_bs + replaced))
         self.assertEquals(a[0].feature_base_first, 1)
-        self.assertEquals(a[0].feature_base_last, len(cassette))
+        self.assertEquals(a[0].feature_base_last, len(replaced))
         self.assertEquals(a[0].feature.strand, 1)
         self.assertEquals(a[0].feature.operation.type, Operation.RECOMBINATION[0])
         self.assertEquals(a[0].feature.operation.genome, c)
@@ -476,10 +497,10 @@ class GenomeRecombinationTest(TestCase):
 
         a = c.fragments.all()[0].indexed_fragment().annotations()
         self.assertEquals(len(a), 1)
-        self.assertEquals(a[0].base_first, len(upstream) + 1)
-        self.assertEquals(a[0].base_last, len(upstream + cassette))
+        self.assertEquals(a[0].base_first, len(upstream) + len(front_bs) + 1)
+        self.assertEquals(a[0].base_last, len(upstream + front_bs + replaced))
         self.assertEquals(a[0].feature_base_first, 1)
-        self.assertEquals(a[0].feature_base_last, len(cassette))
+        self.assertEquals(a[0].feature_base_last, len(replaced))
         # on reverse strand
         self.assertEquals(a[0].feature.strand, -1)
         self.assertEquals(a[0].feature.operation.type, Operation.RECOMBINATION[0])
@@ -505,15 +526,15 @@ class GenomeRecombinationTest(TestCase):
         self.assertNotEqual(g.id, c.id)
         self.assertEquals(
             c.fragments.all()[0].indexed_fragment().sequence,
-            "".join([downstream, upstream, cassette]),
+            "".join([back_bs, downstream, upstream, front_bs, replaced])
         )
 
         a = c.fragments.all()[0].indexed_fragment().annotations()
         self.assertEquals(len(a), 1)
-        self.assertEquals(a[0].base_first, len(downstream + upstream) + 1)
+        self.assertEquals(a[0].base_first, len(back_bs + downstream + upstream + front_bs) + 1)
         self.assertEquals(a[0].base_last, len(downstream + upstream + cassette))
         self.assertEquals(a[0].feature_base_first, 1)
-        self.assertEquals(a[0].feature_base_last, len(cassette))
+        self.assertEquals(a[0].feature_base_last, len(replaced))
         self.assertEquals(a[0].feature.strand, 1)
         self.assertEquals(a[0].feature.operation.type, Operation.RECOMBINATION[0])
         self.assertEquals(a[0].feature.operation.genome, c)
@@ -538,7 +559,7 @@ class GenomeRecombinationTest(TestCase):
         self.assertNotEqual(g.id, c.id)
         self.assertEquals(
             c.fragments.all()[0].indexed_fragment().sequence,
-            "".join([downstream, upstream, cassette]),
+            "".join([front_bs[8:], replaced, back_bs, downstream, upstream, front_bs[0:8]]),
         )
 
     def test_recombine_when_back_arm_is_across_circular_boundary(self):
@@ -561,7 +582,7 @@ class GenomeRecombinationTest(TestCase):
         self.assertNotEqual(g.id, c.id)
         self.assertEquals(
             c.fragments.all()[0].indexed_fragment().sequence,
-            "".join([downstream, upstream, cassette]),
+            "".join([back_bs[8:], downstream, upstream, front_bs, replaced, back_bs[0:8]])
         )
 
     def test_recombines_with_reverse_complement_cassette_correctly(self):
@@ -654,6 +675,22 @@ class GenomeRecombinationTest(TestCase):
             "".join([upstream, cassette, downstream]),
         )
 
+    def test_recombines_genome_parental_sequence_does_nothing(self):
+        upstream = "gagattgtccgcgtttt"
+        front_bs = "catagcgcacaggacgcggag"
+        middle = "cggcacctgtgagccg"
+        back_bs = "taatgaccccgaagcagg"
+        downstream = "gttaaggcgcgaacat"
+
+        template = "".join([upstream, front_bs, middle, back_bs, downstream])
+        cassette = "".join([front_bs, middle, back_bs])
+
+        arm_len = min(len(front_bs), len(back_bs))
+        g = self.build_genome(False, template)
+        c = recombine(g, cassette, arm_len)
+
+        self.assertEqual(c, None)
+
     def test_recombines_multiple_times_on_different_fragments(self):
         upstream = "gagattgtccgcgtttt"
         front_bs = "catagcgcacaggacgcggag"
@@ -673,6 +710,7 @@ class GenomeRecombinationTest(TestCase):
         self.assertEquals(g.fragments.count(), 2)
 
         c = recombine(g, cassette, arm_len)
+
         self.assertEquals(c.fragments.count(), 2)
 
         sequences = [f.indexed_fragment().sequence for f in c.fragments.all()]
@@ -723,6 +761,7 @@ class GenomeRecombinationTest(TestCase):
 
         # first recombination on g, producing first child c
         c1 = recombine(g, cassette, arm_len)
+
         self.assertEquals(c1.fragments.count(), 2)
         sequences = [f.indexed_fragment().sequence for f in c1.fragments.all()]
         sequences = sorted(sequences, key=lambda s: len(s))
@@ -761,6 +800,7 @@ class GenomeRecombinationTest(TestCase):
 
         # second recombination on g, producing second child c2
         c2 = recombine(g, cassette, arm_len)
+
         self.assertEquals(c2.fragments.count(), 2)
         sequences = [f.indexed_fragment().sequence for f in c2.fragments.all()]
         sequences = sorted(sequences, key=lambda s: len(s))
@@ -844,16 +884,19 @@ class GenomeRecombinationTest(TestCase):
         arm_len = min(len(front_bs), len(back_bs))
         g = self.build_genome(True, f)
         c = recombine(g, cassette, arm_len)
+
         self.assertEquals(
             c.fragments.all()[0].indexed_fragment().sequence,
-            downstream
+            back_bs
+            + downstream
             + "t" * 20
             + upstream
             + cassette
             + downstream
             + "c" * 20
             + upstream
-            + cassette,
+            + front_bs
+            + replaced,
         )
 
     def test_multiple_recombines_return_same_child(self):
