@@ -5,7 +5,7 @@ from Bio.Seq import Seq
 from django.test import TestCase
 
 import edge.orfs
-from edge.recombine import find_swap_region_with_annotations, recombine
+from edge.recombine import recombine
 from edge.models import Genome, Fragment, Genome_Fragment
 from edge.blastdb import build_all_genome_dbs, fragment_fasta_fn
 
@@ -27,7 +27,7 @@ class GenomeRecombinationAnnotationsTest(TestCase):
     def setUp(self):
         self.upstream = "gagattgtccgcgtttt"
         self.front_bs = "catagcgcacaggacgcggag"
-        self.middle = "cggcaccttaattgcgaattgcgagctgacgtctgcatgtagccg"
+        self.middle = "cggcaccttaattgcgaattgcgagctgacgtctgcatgtagccc"
         self.back_bs = "taatgaccccgaagcagg"
         self.downstream = "gttaaggcgcgaacat"
         self.template = "".join(
@@ -43,185 +43,6 @@ class GenomeRecombinationAnnotationsTest(TestCase):
     def tearDown(self):
         edge.orfs.min_protein_len = self.old_min_protein_len
 
-    def test_does_not_preserve_annotation_if_replaced_is_different(self):
-        replaced = "aaaaaaaaaaaaaaaaaaa"
-        cassette = "".join([self.front_bs, replaced, self.back_bs])
-
-        self.fragment.annotate(
-            len(self.upstream) + len(self.front_bs),
-            len(self.upstream) + len(self.front_bs) + len(self.middle) - 1,
-            "Foo",
-            "gene",
-            1,
-        )
-
-        r = find_swap_region_with_annotations(self.genome, cassette, self.arm_len)
-        self.assertEquals(len(r), 1)
-        self.assertEquals(len(r[0].cassette_annotations), 0)
-
-    def test_preserves_annotation_with_single_mutation(self):
-        replaced = [c for c in self.middle]
-        self.assertEquals(replaced[10], "a")
-        replaced[10] = "c"
-        replaced = "".join(replaced)
-        cassette = "".join([self.front_bs, replaced, self.back_bs])
-
-        self.fragment.annotate(
-            len(self.upstream) + len(self.front_bs) + 1,
-            len(self.upstream) + len(self.front_bs) + len(self.middle),
-            "Foo",
-            "gene",
-            1,
-        )
-
-        r = find_swap_region_with_annotations(self.genome, cassette, self.arm_len)
-        self.assertEquals(len(r), 1)
-        self.assertEquals(len(r[0].cassette_annotations), 1)
-        a = r[0].cassette_annotations[0]
-        self.assertEquals(a["base_first"], len(self.front_bs) + 1)
-        self.assertEquals(a["base_last"], len(self.front_bs) + len(self.middle))
-        self.assertEquals(a["feature_name"], "Foo A11C")
-
-    def test_detects_insertion(self):
-        replaced = self.middle[0:10] + "c" + self.middle[10:]
-        self.assertEquals(replaced[10], "c")
-        cassette = "".join([self.front_bs, replaced, self.back_bs])
-
-        self.fragment.annotate(
-            len(self.upstream) + len(self.front_bs) + 1,
-            len(self.upstream) + len(self.front_bs) + len(self.middle),
-            "Foo",
-            "gene",
-            1,
-        )
-
-        r = find_swap_region_with_annotations(self.genome, cassette, self.arm_len)
-        self.assertEquals(len(r), 1)
-        self.assertEquals(len(r[0].cassette_annotations), 1)
-        a = r[0].cassette_annotations[0]
-        self.assertEquals(a["base_first"], len(self.front_bs) + 1)
-        self.assertEquals(a["base_last"], len(self.front_bs) + len(self.middle) + 1)
-        self.assertEquals(a["feature_name"], "Foo +11C")
-
-    def test_detects_deletion(self):
-        replaced = self.middle[0:13] + self.middle[14:]
-        cassette = "".join([self.front_bs, replaced, self.back_bs])
-
-        self.fragment.annotate(
-            len(self.upstream) + len(self.front_bs) + 1,
-            len(self.upstream) + len(self.front_bs) + len(self.middle),
-            "Foo",
-            "gene",
-            1,
-        )
-
-        r = find_swap_region_with_annotations(self.genome, cassette, self.arm_len)
-        self.assertEquals(len(r), 1)
-        self.assertEquals(len(r[0].cassette_annotations), 1)
-        a = r[0].cassette_annotations[0]
-        self.assertEquals(a["base_first"], len(self.front_bs) + 1)
-        self.assertEquals(a["base_last"], len(self.front_bs) + len(self.middle) - 1)
-        self.assertEquals(a["feature_name"], "Foo -14G")
-
-    def test_preserves_unchanged_annotation(self):
-        replaced = self.middle[0:10] + self.middle[11:]
-        cassette = "".join([self.front_bs, replaced, self.back_bs])
-
-        self.fragment.annotate(
-            len(self.upstream) + len(self.front_bs) + 12,
-            len(self.upstream) + len(self.front_bs) + len(self.middle) - 1,
-            "Foo",
-            "gene",
-            1,
-        )
-
-        r = find_swap_region_with_annotations(self.genome, cassette, self.arm_len)
-        self.assertEquals(len(r), 1)
-        self.assertEquals(len(r[0].cassette_annotations), 1)
-        a = r[0].cassette_annotations[0]
-        self.assertEquals(a["base_first"], len(self.front_bs) + 12 - 1)
-        self.assertEquals(a["base_last"], len(self.front_bs) + len(self.middle) - 1 - 1)
-        self.assertEquals(a["feature_name"], "Foo")
-
-    def test_preserves_feature_direction_and_type(self):
-        replaced = self.middle[0:13] + self.middle[14:]
-        cassette = "".join([self.front_bs, replaced, self.back_bs])
-
-        self.fragment.annotate(
-            len(self.upstream) + len(self.front_bs) + 1,
-            len(self.upstream) + len(self.front_bs) + len(self.middle),
-            "Foo",
-            "foobar",
-            -1,
-        )
-
-        r = find_swap_region_with_annotations(self.genome, cassette, self.arm_len)
-        self.assertEquals(len(r), 1)
-        self.assertEquals(len(r[0].cassette_annotations), 1)
-        a = r[0].cassette_annotations[0]
-        self.assertEquals(a["base_first"], len(self.front_bs) + 1)
-        self.assertEquals(a["base_last"], len(self.front_bs) + len(self.middle) - 1)
-        self.assertEquals(a["feature_name"], "Foo -14G")
-        self.assertEquals(a["feature_type"], "foobar")
-        self.assertEquals(a["feature_strand"], -1)
-
-    def test_preserves_multiple_annotations(self):
-        replaced = self.middle[0:13] + self.middle[14:]
-        cassette = "".join([self.front_bs, replaced, self.back_bs])
-
-        self.fragment.annotate(
-            len(self.upstream) + 2, len(self.upstream) + 10, "Bar", "static", -1
-        )
-        self.fragment.annotate(
-            len(self.upstream) + len(self.front_bs) + 1,
-            len(self.upstream) + len(self.front_bs) + len(self.middle),
-            "Foo",
-            "changed",
-            1,
-        )
-
-        r = find_swap_region_with_annotations(self.genome, cassette, self.arm_len)
-        self.assertEquals(len(r), 1)
-        self.assertEquals(len(r[0].cassette_annotations), 2)
-
-        a = r[0].cassette_annotations[0]
-        self.assertEquals(a["base_first"], 2)
-        self.assertEquals(a["base_last"], 10)
-        self.assertEquals(a["feature_name"], "Bar")
-        self.assertEquals(a["feature_type"], "static")
-        self.assertEquals(a["feature_strand"], -1)
-
-        a = r[0].cassette_annotations[1]
-        self.assertEquals(a["base_first"], len(self.front_bs) + 1)
-        self.assertEquals(a["base_last"], len(self.front_bs) + len(self.middle) - 1)
-        self.assertEquals(a["feature_name"], "Foo -14G")
-        self.assertEquals(a["feature_type"], "changed")
-        self.assertEquals(a["feature_strand"], 1)
-
-    def test_computes_annotation_for_reverse_cassette(self):
-        replaced = [c for c in self.middle]
-        self.assertEquals(replaced[10], "a")
-        replaced[10] = "c"
-        replaced = "".join(replaced)
-        cassette = "".join([self.front_bs, replaced, self.back_bs])
-        cassette = str(Seq(cassette).reverse_complement())
-
-        self.fragment.annotate(
-            len(self.upstream) + len(self.front_bs) + 1,
-            len(self.upstream) + len(self.front_bs) + len(self.middle),
-            "Foo",
-            "gene",
-            1,
-        )
-
-        r = find_swap_region_with_annotations(self.genome, cassette, self.arm_len)
-        self.assertEquals(len(r), 1)
-        self.assertEquals(len(r[0].cassette_annotations), 1)
-        a = r[0].cassette_annotations[0]
-        self.assertEquals(a["base_first"], len(self.front_bs) + 1)
-        self.assertEquals(a["base_last"], len(self.front_bs) + len(self.middle))
-        self.assertEquals(a["feature_name"], "Foo A11C")
-
     def test_returns_new_orf(self):
         replaced = "atgatcatcatcatcatcatcatcatcatcatcatcatcatcatcatctag"
         cassette = "".join([self.front_bs, replaced, self.back_bs])
@@ -234,19 +55,25 @@ class GenomeRecombinationAnnotationsTest(TestCase):
             1,
         )
 
-        r = find_swap_region_with_annotations(self.genome, cassette, self.arm_len)
-        self.assertEquals(len(r), 1)
-        self.assertEquals(len(r[0].cassette_annotations), 1)
+        c = recombine(self.genome, cassette, self.arm_len)
 
-        a = r[0].cassette_annotations[0]
-        self.assertEquals(a["base_first"], len(self.front_bs) + 1)
-        self.assertEquals(a["base_last"], len(self.front_bs) + len(replaced))
-        self.assertEquals(a["feature_name"], "ORF frame 1")
-        self.assertEquals(a["feature_type"], "ORF")
-        self.assertEquals(a["feature_strand"], 1)
+        f = c.fragments.all()[0].indexed_fragment()
 
-    def test_finds_orf_in_reverse_direction(self):
-        replaced = "atgatcatcatcatcatcatcatcatcatcatcatcatcatcatcatctag"
+        annotations = f.annotations()
+        self.assertEquals(len(annotations), 2)
+
+        a = annotations[0]
+        self.assertEquals(a.feature.type, "operation")
+
+        a = annotations[1]
+        self.assertEquals(a.base_first, len(self.upstream + self.front_bs) + 1)
+        self.assertEquals(a.base_last, len(self.upstream + self.front_bs + replaced))
+        self.assertEquals(a.feature.name, "ORF frame 1")
+        self.assertEquals(a.feature.type, "ORF")
+        self.assertEquals(a.feature.strand, 1)
+
+    def test_returns_new_orf_in_reverse(self):
+        replaced = "atgatcatcatcatcatcatcatcatcatcatcatcatcatcatcatctaa"
         replaced = str(Seq(replaced).reverse_complement())
         cassette = "".join([self.front_bs, replaced, self.back_bs])
 
@@ -258,18 +85,24 @@ class GenomeRecombinationAnnotationsTest(TestCase):
             1,
         )
 
-        r = find_swap_region_with_annotations(self.genome, cassette, self.arm_len)
-        self.assertEquals(len(r), 1)
-        self.assertEquals(len(r[0].cassette_annotations), 1)
+        c = recombine(self.genome, cassette, self.arm_len)
 
-        a = r[0].cassette_annotations[0]
-        self.assertEquals(a["base_first"], len(self.front_bs) + 1)
-        self.assertEquals(a["base_last"], len(self.front_bs) + len(replaced))
-        self.assertEquals(a["feature_name"], "ORF frame 1")
-        self.assertEquals(a["feature_type"], "ORF")
-        self.assertEquals(a["feature_strand"], -1)
+        f = c.fragments.all()[0].indexed_fragment()
 
-    def test_adds_multiple_annotations_to_modified_genome(self):
+        annotations = f.annotations()
+        self.assertEquals(len(annotations), 2)
+
+        a = annotations[0]
+        self.assertEquals(a.feature.type, "operation")
+
+        a = annotations[1]
+        self.assertEquals(a.base_first, len(self.upstream + self.front_bs) + 1)
+        self.assertEquals(a.base_last, len(self.upstream + self.front_bs + replaced))
+        self.assertEquals(a.feature.name, "ORF frame 1")
+        self.assertEquals(a.feature.type, "ORF")
+        self.assertEquals(a.feature.strand, -1)
+
+    def test_annotates_correctly_when_one_bp_is_removed(self):
         replaced = self.middle[0:13] + self.middle[14:]
         cassette = "".join([self.front_bs, replaced, self.back_bs])
 
@@ -285,31 +118,83 @@ class GenomeRecombinationAnnotationsTest(TestCase):
         )
 
         c = recombine(self.genome, cassette, self.arm_len)
+
         f = c.fragments.all()[0].indexed_fragment()
 
         annotations = f.annotations()
         self.assertEquals(len(annotations), 3)
 
         a = annotations[0]
-        self.assertEquals(a.feature.type, "operation")
-
-        a = annotations[1]
         self.assertEquals(a.feature.name, "Bar")
         self.assertEquals(a.feature.type, "static")
         self.assertEquals(a.feature.strand, -1)
         self.assertEquals(a.base_first, len(self.upstream) + 2)
         self.assertEquals(a.base_last, len(self.upstream) + 10)
 
-        a = annotations[2]
-        self.assertEquals(a.feature.name, "Foo -14G")
+        a = annotations[1]
+        self.assertEquals(a.feature.name, "Foo")
         self.assertEquals(a.feature.type, "changed")
         self.assertEquals(a.feature.strand, 1)
-        self.assertEquals(a.base_first, len(self.upstream) + len(self.front_bs) + 1)
-        self.assertEquals(
-            a.base_last, len(self.upstream) + len(self.front_bs) + len(self.middle) - 1
+        self.assertEquals(a.base_first, len(self.upstream + self.front_bs) + 1)
+        self.assertEquals(a.base_last, len(self.upstream + self.front_bs) + 13)
+        self.assertEquals(a.feature_base_first, 1)
+        self.assertEquals(a.feature_base_last, 13)
+
+        a = annotations[2]
+        self.assertEquals(a.feature.name, "Foo")
+        self.assertEquals(a.feature.type, "changed")
+        self.assertEquals(a.feature.strand, 1)
+        self.assertEquals(a.base_first, len(self.upstream + self.front_bs) + 14)
+        self.assertEquals(a.base_last, len(self.upstream + self.front_bs) + 44)
+        self.assertEquals(a.feature_base_first, 15)
+        self.assertEquals(a.feature_base_last, 45)
+
+    def test_adds_single_snp_change_splits_original_annotation(self):
+        replaced = [c for c in self.middle]
+        self.assertEquals(replaced[10], "a")
+        replaced[10] = "c"
+        replaced = "".join(replaced)
+        cassette = "".join([self.front_bs, replaced, self.back_bs])
+
+        self.fragment.annotate(
+            len(self.upstream) + len(self.front_bs) + 1,
+            len(self.upstream) + len(self.front_bs) + len(self.middle),
+            "Foo",
+            "gene",
+            1,
         )
 
-    def test_adds_annotations_correctly_with_reverse_cassette(self):
+        c = recombine(self.genome, cassette, self.arm_len)
+
+        f = c.fragments.all()[0].indexed_fragment()
+
+        annotations = f.annotations()
+        self.assertEquals(len(annotations), 3)
+        for a in annotations:
+            print(str(a))
+
+        a = annotations[0]
+        self.assertEquals(a.feature.name, "Foo")
+        self.assertEquals(a.feature.type, "gene")
+        self.assertEquals(a.feature.strand, 1)
+        self.assertEquals(a.base_first, len(self.upstream) + len(self.front_bs) + 1)
+        self.assertEquals(a.base_last, len(self.upstream) + len(self.front_bs) + 10)
+        self.assertEquals(a.feature_base_first, 1)
+        self.assertEquals(a.feature_base_last, 10)
+
+        a = annotations[1]
+        self.assertEquals(a.feature.type, "operation")
+
+        a = annotations[2]
+        self.assertEquals(a.feature.name, "Foo")
+        self.assertEquals(a.feature.type, "gene")
+        self.assertEquals(a.feature.strand, 1)
+        self.assertEquals(a.base_first, len(self.upstream) + len(self.front_bs) + 12)
+        self.assertEquals(a.base_last, len(self.upstream) + len(self.front_bs) + 45)
+        self.assertEquals(a.feature_base_first, 12)
+        self.assertEquals(a.feature_base_last, 45)
+
+    def test_adds_single_snp_change_splits_original_annotation_when_cassette_is_reversed(self):
         replaced = [c for c in self.middle]
         self.assertEquals(replaced[10], "a")
         replaced[10] = "c"
@@ -326,24 +211,65 @@ class GenomeRecombinationAnnotationsTest(TestCase):
         )
 
         c = recombine(self.genome, cassette, self.arm_len)
+
+        f = c.fragments.all()[0].indexed_fragment()
+
+        annotations = f.annotations()
+        self.assertEquals(len(annotations), 3)
+        for a in annotations:
+            print(str(a))
+
+        a = annotations[0]
+        self.assertEquals(a.feature.name, "Foo")
+        self.assertEquals(a.feature.type, "gene")
+        self.assertEquals(a.feature.strand, 1)
+        self.assertEquals(a.base_first, len(self.upstream) + len(self.front_bs) + 1)
+        self.assertEquals(a.base_last, len(self.upstream) + len(self.front_bs) + 10)
+        self.assertEquals(a.feature_base_first, 1)
+        self.assertEquals(a.feature_base_last, 10)
+
+        a = annotations[1]
+        self.assertEquals(a.feature.type, "operation")
+
+        a = annotations[2]
+        self.assertEquals(a.feature.name, "Foo")
+        self.assertEquals(a.feature.type, "gene")
+        self.assertEquals(a.feature.strand, 1)
+        self.assertEquals(a.base_first, len(self.upstream) + len(self.front_bs) + 12)
+        self.assertEquals(a.base_last, len(self.upstream) + len(self.front_bs) + 45)
+        self.assertEquals(a.feature_base_first, 12)
+        self.assertEquals(a.feature_base_last, 45)
+
+    def test_preserves_annotations_on_homology_arm_fwd_when_inserting_sequence(self):
+        cassette = "".join([self.front_bs, "aaa", self.back_bs])
+
+        # add annotaiton on upstream arm
+        self.fragment.annotate(
+            len(self.upstream) + 2,
+            len(self.upstream) + len(self.front_bs),
+            "Up arm",
+            "feature",
+            1,
+        )
+
+        c = recombine(self.genome, cassette, self.arm_len)
+
         f = c.fragments.all()[0].indexed_fragment()
 
         annotations = f.annotations()
         self.assertEquals(len(annotations), 2)
 
         a = annotations[0]
-        self.assertEquals(a.feature.type, "operation")
+        self.assertEquals(a.feature.name, "Up arm")
+        self.assertEquals(a.feature.type, "feature")
+        self.assertEquals(a.feature.strand, 1)
+        self.assertEquals(a.base_first, len(self.upstream) + 2)
+        self.assertEquals(a.base_last, len(self.upstream) + len(self.front_bs))
 
         a = annotations[1]
-        self.assertEquals(a.feature.name, "Foo A11C")
-        self.assertEquals(a.feature.type, "gene")
-        self.assertEquals(a.feature.strand, 1)
-        self.assertEquals(a.base_first, len(self.upstream) + len(self.front_bs) + 1)
-        self.assertEquals(
-            a.base_last, len(self.upstream) + len(self.front_bs) + len(self.middle)
-        )
+        self.assertEquals(a.feature.type, "operation")
 
-    def test_preserves_annotations_on_homology_arm_fwd_that_have_not_changed(self):
+    def test_preserves_annotations_on_homology_arm_fwd_when_doing_ko(self):
         cassette = "".join([self.front_bs, self.back_bs])
 
         # add annotaiton on upstream arm
@@ -356,22 +282,20 @@ class GenomeRecombinationAnnotationsTest(TestCase):
         )
 
         c = recombine(self.genome, cassette, self.arm_len)
+
         f = c.fragments.all()[0].indexed_fragment()
 
         annotations = f.annotations()
-        self.assertEquals(len(annotations), 2)
+        self.assertEquals(len(annotations), 1)
 
         a = annotations[0]
-        self.assertEquals(a.feature.type, "operation")
-
-        a = annotations[1]
         self.assertEquals(a.feature.name, "Up arm")
         self.assertEquals(a.feature.type, "feature")
         self.assertEquals(a.feature.strand, 1)
         self.assertEquals(a.base_first, len(self.upstream) + 2)
         self.assertEquals(a.base_last, len(self.upstream) + len(self.front_bs))
 
-    def test_preserves_annotations_on_homology_arm_rev_that_have_not_changed(self):
+    def test_preserves_annotations_on_homology_arm_rev(self):
         cassette = "".join([self.front_bs, self.back_bs])
 
         # add annotaiton on reverse strand of downstream arm
@@ -387,15 +311,13 @@ class GenomeRecombinationAnnotationsTest(TestCase):
         )
 
         c = recombine(self.genome, cassette, self.arm_len)
+
         f = c.fragments.all()[0].indexed_fragment()
 
         annotations = f.annotations()
-        self.assertEquals(len(annotations), 2)
+        self.assertEquals(len(annotations), 1)
 
         a = annotations[0]
-        self.assertEquals(a.feature.type, "operation")
-
-        a = annotations[1]
         self.assertEquals(a.feature.name, "Down arm")
         self.assertEquals(a.feature.type, "feature")
         self.assertEquals(a.feature.strand, -1)
@@ -461,6 +383,7 @@ class GenomeRecombinationAnnotationsTest(TestCase):
         ]
 
         c = recombine(self.genome, cassette, self.arm_len, annotations=annotations)
+
         f = c.fragments.all()[0].indexed_fragment()
         fragment_sequence = f.sequence
 
@@ -940,6 +863,7 @@ class GenomeRecombinationAnnotationsTest(TestCase):
         ]
 
         c = recombine(self.genome, cassette, self.arm_len, annotations=annotations)
+
         f = c.fragments.all()[0].indexed_fragment()
         fragment_sequence = f.sequence
 
@@ -1087,6 +1011,7 @@ class GenomeRecombinationAnnotationsTest(TestCase):
         ]
 
         c = recombine(self.genome, cassette, self.arm_len, annotations=annotations)
+
         f = c.fragments.all()[0].indexed_fragment()
         fragment_sequence = f.sequence
 
@@ -1240,6 +1165,7 @@ class GenomeRecombinationAnnotationsTest(TestCase):
         ]
 
         c = recombine(self.genome, cassette, self.arm_len, annotations=annotations)
+
         f = c.fragments.all()[0].indexed_fragment()
         fragment_sequence = f.sequence
 
@@ -1327,3 +1253,147 @@ class GenomeRecombinationAnnotationsTest(TestCase):
         self.assertEquals(
             fragment_sequence[ans[6].base_first - 1 : ans[6].base_last], "t" * 100
         )
+
+    def test_adds_annotations_correctly_when_bps_flanking_new_seq_matches_wt_genome_sequence(
+        self,
+    ):
+        donor = self.middle[:3] + "a" * 97 + "c" * 100 + "t" * 97 + self.middle[-3:]
+        cassette = "".join([self.front_bs, donor, self.back_bs])
+        flen = len(self.front_bs)
+
+        annotations = [
+            dict(
+                base_first=flen + 1,
+                base_last=flen + 100,
+                name="pFavorite",
+                type="promoter",
+                strand=1,
+                qualifiers=None,
+            ),
+            dict(
+                base_first=flen + 101,
+                base_last=flen + 200,
+                name="Favorite",
+                type="gene",
+                strand=1,
+                qualifiers=dict(locus="Favorite", product="Favoritep"),
+            ),
+            dict(
+                base_first=flen + 201,
+                base_last=flen + 300,
+                name="tFavorite",
+                type="terminator",
+                strand=1,
+                qualifiers=None,
+            ),
+        ]
+
+        c = recombine(self.genome, cassette, self.arm_len, annotations=annotations)
+
+        f = c.fragments.all()[0].indexed_fragment()
+        fragment_sequence = f.sequence
+
+        ans = f.annotations()
+        self.assertEquals(len(ans), 4)
+
+        ans = sorted(ans, key=lambda a: (a.base_first, -a.base_last))
+
+        self.assertEquals(ans[0].feature.type, "promoter")
+        self.assertEquals(ans[0].feature.name, "pFavorite")
+        self.assertEquals(ans[0].feature.strand, 1)
+        self.assertEquals(
+            ans[0].base_first, len(self.upstream + self.front_bs) + 1
+        )
+        self.assertEquals(
+            ans[0].base_last, len(self.upstream + self.front_bs) + 100
+        )
+        self.assertEquals(
+            fragment_sequence[ans[0].base_first - 1 : ans[0].base_last],
+            self.middle[:3] + "a" * 97
+        )
+
+        self.assertEquals(ans[1].feature.type, "operation")
+
+        self.assertEquals(ans[2].feature.type, "gene")
+        self.assertEquals(ans[2].feature.name, "Favorite")
+        self.assertEquals(ans[2].feature.strand, 1)
+        self.assertEquals(
+            ans[2].base_first, len(self.upstream + self.front_bs) + 100 + 1
+        )
+        self.assertEquals(
+            ans[2].base_last, len(self.upstream + self.front_bs) + 100 + 100
+        )
+        self.assertEquals(
+            fragment_sequence[ans[2].base_first - 1 : ans[2].base_last], "c" * 100
+        )
+
+        self.assertEquals(ans[3].feature.type, "terminator")
+        self.assertEquals(ans[3].feature.name, "tFavorite")
+        self.assertEquals(ans[3].feature.strand, 1)
+        self.assertEquals(
+            ans[3].base_first, len(self.upstream + self.front_bs) + 200 + 1
+        )
+        self.assertEquals(
+            ans[3].base_last, len(self.upstream + self.front_bs) + 200 + 100
+        )
+        self.assertEquals(
+            fragment_sequence[ans[3].base_first - 1 : ans[3].base_last],
+            "t" * 97 + self.middle[-3:]
+        )
+
+    def test_adds_new_annotations_on_homology_arm(
+        self,
+    ):
+        donor = "a" * 100 + "c" * 100 + "t" * 100
+        cassette = "".join([self.front_bs, donor, self.back_bs])
+
+        annotations = [
+            dict(
+                base_first=1,
+                base_last=3,
+                name="foo",
+                type="feature",
+                strand=1,
+                qualifiers=None,
+            ),
+            dict(
+                base_first=len(self.front_bs + donor + self.back_bs) - 4,
+                base_last=len(self.front_bs + donor + self.back_bs),
+                name="bar",
+                type="feature",
+                strand=-1,
+                qualifiers=None
+            )
+        ]
+
+        c = recombine(self.genome, cassette, self.arm_len, annotations=annotations)
+
+        f = c.fragments.all()[0].indexed_fragment()
+        fragment_sequence = f.sequence
+
+        ans = f.annotations()
+        self.assertEquals(len(ans), 3)
+
+        ans = sorted(ans, key=lambda a: (a.base_first, -a.base_last))
+
+        self.assertEquals(ans[0].feature.type, "feature")
+        self.assertEquals(ans[0].feature.name, "foo")
+        self.assertEquals(ans[0].feature.strand, 1)
+        self.assertEquals(ans[0].base_first, len(self.upstream) + 1)
+        self.assertEquals(ans[0].base_last, len(self.upstream) + 3)
+        self.assertEquals(
+            fragment_sequence[ans[0].base_first - 1 : ans[0].base_last],
+            self.front_bs[:3]
+        )
+
+        self.assertEquals(ans[1].feature.type, "operation")
+
+        self.assertEquals(ans[2].feature.type, "feature")
+        self.assertEquals(ans[2].feature.name, "bar")
+        self.assertEquals(ans[2].feature.strand, -1)
+        self.assertEquals(ans[2].base_first, len(self.upstream + cassette) - 4)
+        self.assertEquals(ans[2].base_last, len(self.upstream + cassette))
+        self.assertEquals(
+            fragment_sequence[ans[2].base_first - 1 : ans[2].base_last],
+            self.back_bs[-5:]
+        )
